@@ -1,10 +1,19 @@
 import io
-from concurrent.futures import Future, ThreadPoolExecutor
+import subprocess  # nosec
+import sys
+from concurrent.futures import ThreadPoolExecutor
 from contextlib import contextmanager
 from shutil import copyfileobj
-from typing import Optional, Tuple, IO, Union, Any, Callable
-
-import subprocess  # nosec
+from typing import (
+    Optional,
+    Tuple,
+    Sequence,
+    IO,
+    Union,
+    Any,
+    Callable,
+    Generator,
+)
 
 # Here bandit warns about:
 # B404:blacklist import_subprocess
@@ -13,7 +22,11 @@ import subprocess  # nosec
 # 2. All calls to subprocess start with gpg, so possibly unsanitized input would
 #    only result in a gpg error message
 
-if hasattr(subprocess, "STARTUPINFO"):
+ISource = Union[bytes, IO[bytes], Callable[[IO[bytes]], None]]
+OSource = Union[int, IO[bytes], Callable[[IO[bytes]], None]]
+# TODO: unquote the "subprocess.Popen[bytes]" type hint once we enforce python 3.9 (multiple occurrences)
+
+if sys.platform == "win32":
 
     def startupinfo():
         _startupinfo = subprocess.STARTUPINFO()
@@ -21,50 +34,45 @@ if hasattr(subprocess, "STARTUPINFO"):
         _startupinfo.wShowWindow = subprocess.SW_HIDE
         return _startupinfo
 
-
 else:
 
-    def startupinfo():
-        return None
+    def startupinfo() -> bool:  # Literal[None] not supported by mypy
+        return False
 
 
 class GPGError(Exception):
     """Error class that forwards errors returned by calls to GnuPG."""
 
 
-def raise_from_future(f: Future):
-    e = f.exception()
+def raise_if_not_none(e: Optional[BaseException]) -> None:
     if e is not None:
         raise e
 
 
-def cmd(command: Tuple[str, ...], out: Optional[IO[bytes]] = None, **kwargs):
-    if out is not None:
-        return cmd_devnull(command, stdout=out, **kwargs)
-    return cmd_pipe_stdout(command, **kwargs)
-
-
 def cmd_devnull(
-    command: Tuple[str, ...], src=None, stdout=subprocess.DEVNULL, **kwargs
-):
+    command: Tuple[str, ...],
+    src: Optional[ISource] = None,
+    stdout: OSource = subprocess.DEVNULL,
+    **kwargs: Any,
+) -> None:
     with cmd_pipe(command, src=src, stdout=stdout, **kwargs):
         pass
 
 
 @contextmanager
-def cmd_pipe_stdout(*args, **kwargs):
+def cmd_pipe_stdout(*args: Any, **kwargs: Any) -> Generator[IO[bytes], None, None]:
     with cmd_pipe(*args, **kwargs) as proc:
-        yield proc.stdout
+        yield assert_io(proc.stdout)
 
 
 @contextmanager
 def cmd_pipe(
     command: Tuple[str, ...],
-    src: Optional[Union[bytes, io.FileIO, Callable]] = None,
-    stdout=subprocess.PIPE,
-    passphrase=None,
-    **kwargs,
-):
+    src: Optional[ISource] = None,
+    stdout: OSource = subprocess.PIPE,
+    passphrase: Optional[str] = None,
+    **kwargs: Any,
+) -> Generator["subprocess.Popen[bytes]", None, None]:
     feed = None
     if src is not None:
         if has_fileno(src) and passphrase is None:
@@ -83,34 +91,40 @@ def cmd_pipe(
         command,
         stderr=subprocess.PIPE,
         stdout=stdout,
-        startupinfo=startupinfo(),
+        startupinfo=startupinfo() or None,
         **kwargs,
     ) as proc:
         if proc.stdin is not None:
             if passphrase is not None:
                 proc.stdin.write(passphrase.encode() + b"\n")
                 proc.stdin.flush()
-            if feed is not None and sink is not None:
-                with ThreadPoolExecutor(max_workers=2) as executor:
-                    future_feed = executor.submit(feed, proc.stdin)
-                    future_sink = executor.submit(sink, proc.stdout)
-                    future_feed.add_done_callback(lambda _: proc.stdin.close())  # type: ignore
-                    raise_from_future(future_feed)
-                    raise_from_future(future_sink)
-            elif feed is not None:
-                feed(proc.stdin)
-                proc.stdin.close()
-            elif sink is not None:
+
+            if feed is not None:
+                if sink is not None and proc.stdout is not None:
+                    with ThreadPoolExecutor(max_workers=2) as executor:
+                        future_feed = executor.submit(feed, proc.stdin)
+                        future_sink = executor.submit(sink, proc.stdout)
+                        future_feed.add_done_callback(
+                            lambda _: proc.stdin and proc.stdin.close()
+                        )
+                        raise_if_not_none(future_feed.exception())
+                        raise_if_not_none(future_sink.exception())
+                else:
+                    feed(proc.stdin)
+                    proc.stdin.close()
+
+            elif sink is not None and proc.stdout is not None:
                 sink(proc.stdout)
+
         yield proc
         err = proc.stderr.read() if proc.stderr is not None else b""
     if proc.returncode != 0:
         raise GPGError(err.decode("utf-8", "replace"))
 
 
-def stderr_lookahead(proc: subprocess.Popen) -> bytes:
+def stderr_lookahead(proc: "subprocess.Popen[bytes]") -> bytes:
     if proc.stderr is not None:
-        out = proc.stderr.read()
+        out: bytes = proc.stderr.read()
         proc.stderr.close()
         proc.stderr = io.BytesIO(out)
     else:
@@ -119,17 +133,17 @@ def stderr_lookahead(proc: subprocess.Popen) -> bytes:
 
 
 class ExpectProc:
-    def __init__(self, proc):
+    def __init__(self, proc: "subprocess.Popen[bytes]"):
         self.proc = proc
-        self.source = proc.stderr
-        self.dest = proc.stdin
+        self.source = assert_io(proc.stderr, name="stderr")
+        self.dest = assert_io(proc.stdin, name="stdin")
         self.stdout = proc.stdout
 
-    def expect(self, expected, prefix=b"[GNUPG:] GET_"):
+    def expect(self, expected: bytes, prefix: bytes = b"[GNUPG:] GET_") -> None:
         actual = b""
         while not actual.startswith(prefix):
             actual = self.source.read(len(prefix))
-            c = True
+            c = b"1"
             while c and c != b"\n":
                 c = self.source.read(1)
                 if c != b"\r":  # Skip '\r' (windows)
@@ -137,15 +151,17 @@ class ExpectProc:
             if not actual:
                 raise ValueError("Unexpected end of source")
         if actual != expected:
-            raise ValueError(f"Unexpected prompt from gpg:\n{expected}\n{actual}")
+            raise ValueError(
+                f"Unexpected prompt from gpg:\n{expected.decode()}\n{actual.decode()}"
+            )
 
-    def put(self, msg):
+    def put(self, msg: bytes) -> None:
         self.dest.write(msg)
         self.dest.flush()
 
 
 @contextmanager
-def expect(command, **kwargs):
+def expect(command: Sequence[str], **kwargs: Any) -> Generator[ExpectProc, None, None]:
     # For the bandit vulnerability suppressed here (nosec):
     # see info close the subprocess import
     with subprocess.Popen(  # nosec
@@ -158,7 +174,7 @@ def expect(command, **kwargs):
         yield ExpectProc(proc)
 
 
-def feed_from_any(src: Union[bytes, io.FileIO, Callable]) -> Callable:
+def feed_from_any(src: ISource) -> Callable[[IO[bytes]], None]:
     if isinstance(src, bytes):
         return feed_from_str(src)
     if callable(src):
@@ -166,22 +182,22 @@ def feed_from_any(src: Union[bytes, io.FileIO, Callable]) -> Callable:
     return feed_from_stream(src)
 
 
-def feed_from_str(src: bytes) -> Callable:
-    def feed(stdin):
+def feed_from_str(src: bytes) -> Callable[[IO[bytes]], None]:
+    def feed(stdin: IO[bytes]) -> None:
         stdin.write(src)
 
     return feed
 
 
-def feed_from_stream(src: io.FileIO) -> Callable:
-    def feed(stdin):
+def feed_from_stream(src: IO[bytes]) -> Callable[[IO[bytes]], None]:
+    def feed(stdin: IO[bytes]) -> None:
         copyfileobj(src, stdin)
 
     return feed
 
 
-def feed_from_callable(src: Callable) -> Callable:
-    def feed(stdin):
+def feed_from_callable(src: Callable[[IO[bytes]], None]) -> Callable[[IO[bytes]], None]:
+    def feed(stdin: IO[bytes]) -> None:
         src(stdin)
 
     return feed
@@ -195,3 +211,9 @@ def has_fileno(s: Any) -> bool:
             return False
         return True
     return False
+
+
+def assert_io(stream: Optional[IO[bytes]], name: str = "stdout") -> IO[bytes]:
+    if stream is None:
+        raise ValueError(f"Unexpected {name} of type NoneType.")
+    return stream
