@@ -1,28 +1,41 @@
 import collections.abc
 import sys
 import warnings
-from typing import Optional, Union, Tuple
+from typing import Optional, Union, Tuple, Dict, TypeVar, Type, Any, Callable, cast
+from abc import ABC
 
 import dataclasses
 
 
-def deserialize(T: type):
+class IsDataclass(type):
+    # TODO: unquote once we enforce python 3.9
+    __dataclass_fields__: Dict[str, "dataclasses.Field[Any]"]
+
+
+class IsTypingType(ABC):
+    __args__: Tuple[type, ...]
+
+
+Tp = TypeVar("Tp")
+
+
+def deserialize(T: Type[Tp]) -> Callable[[Any], Tp]:
     """Creates a deserializer for the type :T:. It handles dataclasses,
     sequences, typing.Optional and primitive types.
 
     :returns: A deserializer, converting a dict, list or primitive to :T:
     """
 
-    def _deserialize(data):
+    def _deserialize(data: Any) -> Tp:
         seq_type = sequence_type(T)
         if seq_type is not None:
             try:
-                item_type = T.__args__[0]
+                item_type = cast(IsTypingType, T).__args__[0]
             except AttributeError as e:
                 raise ValueError(
                     f"Sequence of type {seq_type.__name__} without item type"
                 ) from e
-            return seq_type(map(deserialize(item_type), data))
+            return cast(Tp, seq_type(map(deserialize(item_type), data)))
         if dataclasses.is_dataclass(T):
             fields = dataclasses.fields(T)
             unexpected_keys = set(data.keys()) - set(f.name for f in fields)
@@ -33,21 +46,22 @@ def deserialize(T: type):
             converted_data = {
                 f.name: deserialize(f.type)(data.get(f.name, f.default)) for f in fields
             }
-            return T(**converted_data)
+            return cast(Tp, cast(IsDataclass, T)(**converted_data))
         opt_type = optional_type(T)
         if opt_type is not None:
-            if data is None:
-                return None
-            return opt_type(data)
+            return cast(Tp, None if data is None else opt_type(data))
         dct_type = dict_type(T)
         if dct_type is not None:
             key_type, val_type = dct_type
-            return {
-                deserialize(key_type)(key): deserialize(val_type)(val)
-                for key, val in data.items()
-            }
+            return cast(
+                Tp,
+                {
+                    deserialize(key_type)(key): deserialize(val_type)(val)
+                    for key, val in data.items()
+                },
+            )
 
-        return T(data)
+        return cast(Tp, cast(type, T)(data))
 
     return _deserialize
 
@@ -63,9 +77,9 @@ def optional_type(T: type) -> Optional[type]:
     except ValueError:
         return None
     if isinstance(None, T1):
-        return T2
+        return cast(type, T2)
     if isinstance(None, T2):
-        return T1
+        return cast(type, T1)
     return None
 
 
