@@ -1,3 +1,5 @@
+# pylint: disable=too-many-lines
+
 import io
 import platform
 import os
@@ -8,16 +10,17 @@ from typing import (
     Optional,
     List,
     Tuple,
+    Sequence,
     Iterator,
     Iterable,
     Union,
-    Sequence,
     Any,
     Type,
     IO,
     Dict,
     ContextManager,
     Callable,
+    AbstractSet,
 )
 import urllib.request
 
@@ -29,9 +32,12 @@ from .cmd import (
     cmd_devnull,
     cmd_pipe,
     cmd_pipe_stdout,
+    assert_io,
     expect,
     GPGError,
     stderr_lookahead,
+    ISource,
+    OSource,
 )
 from .model import (
     Uid,
@@ -41,17 +47,22 @@ from .model import (
     TrustModel,
     SignatureValidity,
     CompressAlgo,
-    ColonRecordBase,
     Signature,
     RevocationSignature,
+    Validity,
+    KeyCapability,
+    SpecialKeyCapability,
+)
+from .keyserver import (
+    UrlOpener,
+    VksUploadResponse,
+    download_key,
+    upload_keys,
+    vks_get_key_by_any_identifier,
+    vks_upload_key,
 )
 
-from .keyserver import upload_keys, download_key
-
-
-def parse_data_class(T: type, args: Sequence[Any]):
-    fields = (f.metadata.get("deserialize", f.type) for f in dataclass_fields(T))
-    return T(*(deserialize(f_type)(arg) for f_type, arg in zip(fields, args)))
+Token = Union["ColonRecordBase", "Factory", Optional[Uid], str, Signature, list]
 
 
 unimplemented_col_rec_ids = (
@@ -68,73 +79,112 @@ unimplemented_col_rec_ids = (
     "cfg",  # Configuration data [*]
 )
 
-col_rec_types = {}
+col_rec_types: Dict[str, Type["ColonRecordBase"]] = {}
 
 
-def col_rec_type(identifier):
-    def decorator(cls):
+def col_rec_type(
+    identifier: str,
+) -> Callable[[Type["ColonRecordBase"]], Type["ColonRecordBase"]]:
+    def decorator(cls: Type["ColonRecordBase"]) -> Type["ColonRecordBase"]:
         col_rec_types[identifier] = cls
         return cls
 
     return decorator
 
 
-def extract_key_fields(rec):
-    fields = (
-        "validity",
-        "key_length",
-        "pub_key_algorithm",
-        "key_id",
-        "creation_date",
-        "expiration_date",
-        "owner_trust",
-        "key_capabilities",
-        "origin",
-    )
-    return {
-        key: val
-        for key, val in ((f, getattr(rec, f, None)) for f in fields)
-        if val is not None
-    }
+AnyKeyCapability = Union[KeyCapability, SpecialKeyCapability]
+
+
+@dataclass
+class ColonRecordBase:
+    """According to https://github.com/gpg/gnupg/blob/master/doc/DETAILS"""
+
+    validity: Optional[Validity] = None  # 2
+    key_length: Optional[int] = None  # 3
+    pub_key_algorithm: Optional[int] = None
+    key_id: Optional[str] = None  # 5
+    creation_date: Optional[str] = None
+    expiration_date: Optional[str] = None
+    cert_S_N_uid_hash_trust_signature_info: Optional[str] = None
+    owner_trust: Optional[str] = None
+    user_id: Optional[str] = None  # 10
+    signature_class: Optional[str] = None
+    key_capabilities: Optional[str] = None
+    issuer: Optional[str] = None  # 13: Issuer certificate fingerprint or other info.
+    flag: Optional[str] = None
+    token_S_N: Optional[str] = None  # 15
+    hash_algorithm: Optional[int] = None  # 16: Hash algorithm. 2 = SHA-1, 8 = SHA-256
+    curve_name: Optional[str] = None
+    compliance_flags: Optional[str] = None
+    last_update: Optional[str] = None
+    origin: Optional[str] = None  # 20
+    comment: Optional[str] = None
+
+    def into(
+        self,
+    ) -> Token:
+        return self
+
+
+class ColonRecordKey(ColonRecordBase):
+    key_type: KeyType = KeyType.public
+
+    def key_fields(self) -> Dict[str, Any]:
+        fields = (
+            "validity",
+            "key_length",
+            "pub_key_algorithm",
+            "key_id",
+            "creation_date",
+            "expiration_date",
+            "owner_trust",
+            "origin",
+        )
+        field_values = ((f, getattr(self, f, None)) for f in fields)
+        return {key: val for key, val in field_values if val is not None}
 
 
 @col_rec_type("sec")
-class ColonRecordSec(ColonRecordBase):
+class ColonRecordSec(ColonRecordKey):
     """Secret key"""
 
-    def into(self):
-        return Factory(key_type=KeyType.secret, **extract_key_fields(self))
+    key_type = KeyType.secret
+
+    def into(self) -> "Factory":
+        return Factory(self)
 
 
 @col_rec_type("pub")
-class ColonRecordPub(ColonRecordBase):
+class ColonRecordPub(ColonRecordKey):
     """Public key"""
 
-    def into(self):
-        return Factory(key_type=KeyType.public, **extract_key_fields(self))
+    def into(self) -> "Factory":
+        return Factory(self)
 
 
 @col_rec_type("sub")
-class ColonRecordSub(ColonRecordBase):
+class ColonRecordSub(ColonRecordKey):
     """Subkey (secondary key)"""
 
-    def into(self):
-        return SubKeyFactory(key_type=KeyType.public, **extract_key_fields(self))
+    def into(self) -> "SubKeyFactory":
+        return SubKeyFactory(self)
 
 
 @col_rec_type("ssb")
-class ColonRecordSsb(ColonRecordBase):
+class ColonRecordSsb(ColonRecordKey):
     """Subkey (secondary key)"""
 
-    def into(self):
-        return SubKeyFactory(key_type=KeyType.secret, **extract_key_fields(self))
+    key_type = KeyType.secret
+
+    def into(self) -> "SubKeyFactory":
+        return SubKeyFactory(self)
 
 
 @col_rec_type("uid")
 class ColonRecordUid(ColonRecordBase):
     """User id"""
 
-    def into(self):
+    def into(self) -> Optional[Uid]:
         return Uid.from_str(self.user_id)
 
 
@@ -142,7 +192,7 @@ class ColonRecordUid(ColonRecordBase):
 class ColonRecordFpr(ColonRecordBase):
     """Fingerprint"""
 
-    def into(self):
+    def into(self) -> str:
         if self.user_id is None:
             raise ValueError("Field 'user_id' missing in record")
         return self.user_id
@@ -157,7 +207,7 @@ class ColonRecordSig(ColonRecordBase):
     # for parsing only.
     validity: Optional[SignatureValidity]  # type: ignore
 
-    def into_dict(self):
+    def into_dict(self) -> Dict[str, Any]:
         uid = Uid.from_str(self.user_id)
         return dict(
             issuer_uid=uid,
@@ -169,7 +219,7 @@ class ColonRecordSig(ColonRecordBase):
             validity=self.validity,
         )
 
-    def into(self):
+    def into(self) -> Signature:
         return Signature(**self.into_dict())
 
 
@@ -178,7 +228,7 @@ class ColonRecordSig(ColonRecordBase):
 class ColonRecordRev(ColonRecordSig):
     """Revocation Signature"""
 
-    def into(self):
+    def into(self) -> RevocationSignature:
         sig = self.into_dict()
         sig_class = sig.pop("signature_class")
         try:
@@ -187,13 +237,13 @@ class ColonRecordRev(ColonRecordSig):
             revocation_reason = None
         return RevocationSignature(
             comment=self.comment,
-            signature_class=sig_class,
             reason=revocation_reason,
             **sig,
+            signature_class=sig_class,
         )
 
 
-def lex_colon(payload: IO[bytes]) -> Iterator:
+def lex_colon(payload: IO[bytes]) -> Iterator[Token]:
     """Tokenizes the output of a call to the gpg command to list keys (payload)
     one line at a time. Works for both commands to list public or secret keys.
     Example of payload:
@@ -221,11 +271,17 @@ def lex_colon(payload: IO[bytes]) -> Iterator:
         record_type, *args = map(
             lambda s: s if s else None, rec.decode("utf-8", "replace").split(":")
         )
-        args = args[: len(dataclass_fields(ColonRecordBase))]
+        if record_type is None:
+            raise RuntimeError(
+                f"Error while parsing row of type {record_type}"
+                f"\nGot line without type"
+                f"\ncolumns: {args}"
+            )
         if record_type in unimplemented_col_rec_ids:
             continue
+        kwargs = dict(zip((f.name for f in dataclass_fields(ColonRecordBase)), args))
         try:
-            token = parse_data_class(col_rec_types[record_type], args).into()
+            token = deserialize(col_rec_types[record_type])(kwargs).into()
         except BaseException as e:
             raise ValueError(
                 f"Error while parsing row of type {record_type}"
@@ -239,37 +295,60 @@ def lex_colon(payload: IO[bytes]) -> Iterator:
 class Factory:
     type_: Type[SubKey] = Key
 
-    def __init__(self, origin: Optional[str] = None, **kwargs):
-        self.kwargs = kwargs
-        if origin:
-            origin = origin.replace("\\x3a", ":").lstrip("0123456789 ")
+    def __init__(self, rec: ColonRecordKey):
+        self.kwargs = rec.key_fields()
+        self.kwargs.update(self.parse_key_capabilities(rec.key_capabilities))
+        self.kwargs["key_type"] = rec.key_type
+        if rec.origin:
+            origin = rec.origin.replace("\\x3a", ":").lstrip("0123456789 ")
             self.kwargs["origin"] = origin or None
 
-    def append(self, key, val):
+    def append(self, key: str, val: Any) -> "Factory":
         self.kwargs[key] = self.kwargs.get(key, ()) + (val,)
         return self
 
-    def append_subkey(self, val):
+    def append_subkey(self, val: SubKey) -> "Factory":
         if self.kwargs["key_type"] != val.key_type:
             raise ValueError(f"Wrong sub key type '{val.key_type}'")
         return self.append("sub_keys", val)
 
-    def set(self, key, val):
+    def set(self, key: str, val: Any) -> "Factory":
         if key in self.kwargs:
             raise ValueError(f"Key '{key}' already set")
         self.kwargs[key] = val
         return self
 
-    def build(self):
+    def build(self) -> SubKey:
         try:
             return self.type_(**self.kwargs)
         except (TypeError, ValueError) as e:
             raise type(e)(e.args[0].replace("__init__", self.type_.__name__))
 
+    @staticmethod
+    def parse_key_capabilities(s: Optional[str]) -> Dict[str, Any]:
+        mapping: Dict[str, AnyKeyCapability] = {e.value[0]: e for e in KeyCapability}
+        mapping.update({e.value[0]: e for e in SpecialKeyCapability})
+
+        if s is None:
+            s = ""
+        return dict(
+            combined_key_capabilities=frozenset(
+                mapping[c.lower()] for c in s if c.isupper() and c != "D"
+            ),
+            key_capabilities=frozenset(mapping[c] for c in s if c.islower()),
+            disabled="D" in s,
+        )
+
 
 class SubKeyFactory(Factory):
     type_ = SubKey
 
+    @staticmethod
+    def parse_key_capabilities(s: Optional[str]) -> Dict[str, Any]:
+        d = Factory.parse_key_capabilities(s)
+        del d["combined_key_capabilities"]
+        return d
+
 
 # Factory, type : rule how to transform subsequences of tokens
 # into reduced token sequences. For example, in:
@@ -292,7 +371,7 @@ key_parser = compile_grammar(
 )
 
 
-def parse_keys(payload: IO[bytes]):
+def parse_keys(payload: IO[bytes]) -> List[Key]:
     return key_parser(list(lex_colon(payload)) + [[]])
 
 
@@ -329,11 +408,14 @@ def get_gnupg_binary(gnupg_binary: Optional[str] = None) -> str:
 
     raise FileNotFoundError(
         f"GnuPG executable [{', '.join(binaries_to_test)}] not found. "
-        "Please make sure that the GnuPG binary is found in your PATH."
+        "Please make sure that the GnuPG binary is found in your PATH. "
+        "If your GnuPG binary has a non-standard name (i.e. different from "
+        "gpg and gpg2), or is located in a directory outside of your PATH, "
+        "please specify it via the 'gnupg_binary' argument of 'GPGStore'."
     )
 
 
-class GPGStore:
+class GPGStore:  # pylint: disable=too-many-public-methods
     """Main class providing the functionality of gpg-lite.
 
     The main class of the library is the representation of a GnuPG 'home'
@@ -351,7 +433,11 @@ class GPGStore:
     def __init__(
         self, gnupg_home_dir: Optional[str] = None, gnupg_binary: Optional[str] = None
     ):
-        self._gnupg_home_dir = gnupg_home_dir
+        self._gnupg_home_dir = (
+            os.path.abspath(os.path.expanduser(gnupg_home_dir))
+            if gnupg_home_dir
+            else None
+        )
         self._gpg_bin = get_gnupg_binary(gnupg_binary)
         self._version = self.version()
         self._pw_args_t: Tuple[str, ...] = ("--passphrase-fd", "0")
@@ -362,20 +448,20 @@ class GPGStore:
             # added in order for the --passphrase-fd option to be used.
             self._pw_args_t = ("--batch",) + self._pw_args_t
 
-    def _pw_args(self, pw):
+    def _pw_args(self, pw: Optional[str]) -> Tuple[str, ...]:
         return self._pw_args_t if pw is not None else ()
 
     @property
-    def gnupg_binary(self):
+    def gnupg_binary(self) -> Optional[str]:
         return shutil.which(self._gpg_bin)
 
     @property
-    def gnupg_home_dir(self):
+    def gnupg_home_dir(self) -> str:
         return self._gnupg_home_dir or get_default_gnupg_home_dir()
 
     @property
-    def _base_args(self):
-        args = (self._gpg_bin,)
+    def _base_args(self) -> Tuple[str, ...]:
+        args: Tuple[str, ...] = (self._gpg_bin,)
         if self._gnupg_home_dir is not None:
             args += ("--homedir", self._gnupg_home_dir)
         return args
@@ -383,7 +469,7 @@ class GPGStore:
     def default_key(self) -> Optional[str]:
         config_file = os.path.join(self.gnupg_home_dir, "gpg.conf")
         try:
-            with open(config_file) as f_cfg:
+            with open(config_file, encoding="utf-8") as f_cfg:
                 default_key = next(
                     line for line in f_cfg if line.startswith("default-key ")
                 )
@@ -461,7 +547,7 @@ class GPGStore:
         args += search_terms
         try:
             with cmd_pipe(self._base_args + args) as proc:
-                return parse_keys(proc.stdout)
+                return parse_keys(assert_io(proc.stdout))
         except GPGError as e:
             if "error reading key" in str(e).lower():
                 return []
@@ -469,15 +555,15 @@ class GPGStore:
 
     def encrypt(
         self,
-        source: Optional[Union[bytes, io.FileIO, Callable]],
-        recipients: List[str],
-        output: Optional[Union[io.FileIO, Callable]],
+        source: Optional[ISource],
+        recipients: Sequence[str],
+        output: Optional[OSource],
         sign: Optional[str] = None,
         passphrase: Optional[str] = None,
         trust_model: TrustModel = TrustModel.pgp,
         compress_algo: Optional[CompressAlgo] = None,
         compress_level: Optional[int] = None,
-    ):
+    ) -> None:
         """Encrypt data with the public PGP key of recipients.
 
         Both source and output can be file like objects or callables
@@ -527,18 +613,19 @@ class GPGStore:
                 )
             args += ("--compress-level", str(compress_level))
 
+        maybe_stdout = {} if output is None else {"stdout": output}
         with cmd_pipe(
             self._base_args + self._pw_args(passphrase) + args,
             src=source,
-            stdout=output,
+            **maybe_stdout,
             passphrase=passphrase,
         ):
             pass
 
     def decrypt(
         self,
-        source: Union[io.FileIO, Callable],
-        output: Union[io.FileIO, Callable],
+        source: ISource,
+        output: Optional[OSource],
         passphrase: Optional[str] = None,
         trust_model: TrustModel = TrustModel.pgp,
     ) -> List[str]:
@@ -552,7 +639,7 @@ class GPGStore:
         :param passphrase: passphrase of the private key to be used to decrypt
             the data.
         :param trust_model: trust model to be followed by gpg.
-        :return: fingerprint or keyid of the signee's key.
+        :return: fingerprint or key ID of the signee's key.
         """
 
         # Note: the '--keyid-format', 'long' argument is only needed for
@@ -566,11 +653,9 @@ class GPGStore:
             trust_model.name,
             "--decrypt",
         )
-        maybe_stdout = {}
-        if output is not None:
-            maybe_stdout["stdout"] = output
+        maybe_stdout = {} if output is None else {"stdout": output}
         with cmd_pipe(
-            self._base_args + self._pw_args(passphrase) + args,  # type: ignore
+            self._base_args + self._pw_args(passphrase) + args,
             **maybe_stdout,
             src=source,
             passphrase=passphrase,
@@ -586,49 +671,95 @@ class GPGStore:
         self,
         full_name: str,
         email: str,
-        passphrase: str,
-        key_length: int = 4096,
+        passphrase: Optional[str],
         key_type: str = "RSA",
+        key_length: int = 4096,
+        key_capabilities: AbstractSet[KeyCapability] = frozenset(
+            (KeyCapability.sign, KeyCapability.certify)
+        ),
+        key_curve: Optional[str] = None,
+        subkey_type: str = "RSA",
+        subkey_length: int = 4096,
+        subkey_capabilities: AbstractSet[KeyCapability] = frozenset(
+            (KeyCapability.encrypt,),
+        ),
+        subkey_curve: Optional[str] = None,
     ) -> str:
         """Create a new public/private PGP key pair.
 
         :param full_name: user name to be used in the key user ID.
         :param email: email to be used in the key user ID.
         :param passphrase: passphrase (password) that will be requested when
-            using the private key.
+            using the private key. Explicitely set to None if you want an
+            unprotected key.
         :param key_length: bit-length of the key to be generated. Generally
             speaking, longer keys are stronger (more difficult to break). For
             "RSA" keys the recommended length is currently 2048 or 4096. Keys
             shorter than 2048 bits should not be used any longer.
-        :param key_type: "RSA" (default) or "DSA". It is strongly recommended
+        :param key_type: "RSA" (default), "DSA" or other supported gpg algorithms
+            (e.g. eddsa, ecdh for ECC keys).
+            It is strongly recommended
             to use "RSA", as these key types are more secure.
+        :param key_capabilities: The available capabilities are defined in the enum :KeyCapability:
+        :param subkey_curve: The requested elliptic curve of the generated key. This is a required parameter for ECC keys. It is ignored for non-ECC keys.
+        :param subkey_type: Type for the subkey (See description for :key_type:).
+        :param subkey_length: Length for subkey (See description for :key_length:)
+        :param subkey_capabilities: See description for :key_capabilities:)
+        :param subkey_curve: Key curve for a subkey; similar to :key_curve:.
         :return: fingerprint of the newly generated key.
+
+        Example: To create a ECC key, use:
+          key_type="eddsa",
+          subkey_type="ecdh",
+          key_curve="Ed25519",
+          subkey_curve="Curve25519"
         """
-        args = ("--batch", "--status-fd", "1", "--gen-key")
+
         with cmd_pipe(
-            self._base_args + args,
-            src=f"""
-%echo Generating a basic OpenPGP key
+            self._base_args + ("--batch", "--gen-key", "--status-fd", "1"),
+            src=(
+                f"""
 Key-Type: {key_type}
 Key-Length: {key_length}
-Subkey-Type: {key_type}
-Subkey-Length: {key_length}
+Subkey-Type: {subkey_type}
+Subkey-Length: {subkey_length}
 Name-Real: {full_name}
 Name-Email: {email}
-Expire-Date: 0
-Passphrase: {passphrase}
+Expire-Date: 0"""
+                + (
+                    f"""
+Key-Usage: {",".join(c.value for c in key_capabilities)}
+Subkey-Usage: {",".join(c.value for c in subkey_capabilities)}"""
+                    if self._version >= (2, 2, 0)
+                    else ""
+                )
+                + f"""
+{"Key-Curve: " + key_curve if key_curve is not None else ""}
+{"Subkey-Curve: " + subkey_curve if subkey_curve is not None else ""}
+{"Passphrase: " + passphrase if passphrase is not None else "%no-protection"}
 %commit
-%echo done
-""".encode(),
+"""
+            ).encode(),
         ) as proc:
-            l = next(
-                line for line in proc.stdout if line.startswith(b"[GNUPG:] KEY_CREATED")
+            stdout_lines = (
+                assert_io(proc.stdout).read().decode("utf-8", "replace").splitlines()
             )
-            return l.rstrip(b"\r\n").split(b" ")[-1].decode("utf-8", "replace")
+            try:
+                l: str = next(
+                    line
+                    for line in stdout_lines
+                    if line.startswith("[GNUPG:] KEY_CREATED")
+                )
+            except StopIteration:
+                raise GPGError(stdout_lines) from None
+            return l.split(" ")[-1]
 
     def send_keys(
-        self, *fingerprints: str, keyserver: str, url_opener=urllib.request.urlopen
-    ):
+        self,
+        *fingerprints: str,
+        keyserver: str,
+        url_opener: UrlOpener = urllib.request.urlopen,
+    ) -> None:
         """Upload keys from the user's local keyring to a keyserver."""
         upload_keys(
             keys_as_ascii_armor=self.export(*fingerprints),
@@ -636,7 +767,13 @@ Passphrase: {passphrase}
             url_opener=url_opener,
         )
 
-    def recv_keys(self, *fingerprints: str, keyserver: str, **kwargs):
+    def vks_send_key(self, fingerprint: str, keyserver: str) -> VksUploadResponse:
+        """Upload a key from the user's local keyring to a Verifying Keyserver"""
+        return vks_upload_key(
+            key=self.export(fingerprint).decode(), keyserver=keyserver
+        )
+
+    def recv_keys(self, *fingerprints: str, keyserver: str, **kwargs: Any) -> None:
         """Download keys from a keyserver to the user's local keyring."""
         for fingerprint in fingerprints:
             with download_key(
@@ -644,6 +781,17 @@ Passphrase: {passphrase}
             ) as key:
                 self.import_file(key.read())
 
+    def vks_recv_key(self, identifier: str, keyserver: str) -> None:
+        """Download a key from a Verifying Keyserver to the user's local keyring
+
+        :param identifier: Fingerprint, Key ID, or email address
+        :param keyserver: keyserver url
+        """
+        with vks_get_key_by_any_identifier(
+            keyserver=keyserver, identifier=identifier
+        ) as key:
+            self.import_file(key.read())
+
     def delete_pub_keys(self, *fingerprints: str) -> None:
         """Delete one or more public keys with the specified fingerprints from
         the local keyring.
@@ -672,9 +820,7 @@ Passphrase: {passphrase}
             + fingerprints
         )
 
-    def import_file(
-        self, source: Union[bytes, IO[bytes]], *, passphrase: Optional[str] = None
-    ):
+    def import_file(self, source: ISource, *, passphrase: Optional[str] = None) -> None:
         cmd_devnull(
             self._base_args + self._pw_args(passphrase) + ("--import", "--batch", "-"),
             src=source,
@@ -691,16 +837,53 @@ Passphrase: {passphrase}
         with cmd_pipe_stdout(cmd) as stream:
             return stream.read()
 
-    def version(self):
+    def export_secret(
+        self, fingerprint: str, passphrase: str, armor: bool = True
+    ) -> bytes:
+        """Extracts a private key from the user's GnuPG keyring and
+        returns it as a byte string.
+
+        :param fingerprint: fingerprint of PGP key to extract.
+        :param passphrase: passphrase for the private key to extract.
+        :param armor: if True, the key is exported in "ascii armored"
+            format. If False, the key is exported in a binary format.
+        :raises GPGError: if the key is not found in the keyring or a
+            bad password was provided.
+        """
+        try:
+            with cmd_pipe_stdout(
+                self._base_args
+                + self._pw_args(passphrase)
+                + (("--armor",) if armor else ())
+                + ("--export-secret-keys", fingerprint),
+                passphrase=passphrase,
+                src=b"",
+            ) as stream:
+                key = stream.read()
+        except GPGError as e:
+            if "passphrase" in str(e).lower():
+                raise GPGError(
+                    f"Unable to export private key: wrong passphrase for the key ({fingerprint})"
+                ) from e
+            raise e
+        if not key:
+            raise GPGError(f"Secret key for {fingerprint} not found")
+        return key
+
+    def version(self) -> Tuple[int, ...]:
         with cmd_pipe(self._base_args + ("--version",)) as proc:
             return tuple(
                 map(
                     int,
-                    proc.stdout.readline().rstrip(b"\n").split(b" ")[-1].split(b"."),
+                    assert_io(proc.stdout)
+                    .readline()
+                    .rstrip(b"\n")
+                    .split(b" ")[-1]
+                    .split(b"."),
                 )
             )
 
-    def gen_revoke(self, fingerprint: str, passphrase) -> bytes:
+    def gen_revoke(self, fingerprint: str, passphrase: str) -> bytes:
         """Generate a revocation certificate for a PGP key with the specified
         fingerprint.
         """
@@ -730,13 +913,13 @@ Passphrase: {passphrase}
                 proc.put(b"y\n")
             except ValueError as e:
                 raise GPGError("Failed to generate a revocation certificate") from e
-            return proc.stdout.read()
+            return assert_io(proc.stdout).read()
 
     def revoke_key(
         self,
         fingerprint: str,
-        revocation_certif: Optional[Union[bytes, IO[bytes]]] = None,
-        passphrase: Optional[str] = None,
+        passphrase: str,
+        revocation_certif: Optional[ISource] = None,
         keyserver: Optional[str] = None,
     ) -> None:
         """Revoke a PGP key, optionally pushing it to a keyserver.
@@ -764,10 +947,10 @@ Passphrase: {passphrase}
 
     def detach_sig(
         self,
-        src: Union[bytes, IO[bytes]],
+        src: ISource,
         passphrase: str,
         signee: Optional[str] = None,
-    ) -> ContextManager:
+    ) -> ContextManager[IO[bytes]]:
         """Generate a detached signature for a given input.
 
         :param src: binary data for which a signature should be generated.
