@@ -4,11 +4,11 @@ from gpg_lite import cmd
 
 
 class TestGPG(unittest.TestCase):
-    def test_cmd_pipe(self):
+    def test_cmd_pipe(self) -> None:
         with self.assertRaises(FileNotFoundError):
             with cmd.cmd_pipe(("non-existing-bin",)) as proc:
-                proc.stdout.read()
+                cmd.assert_io(proc.stdout).read()
 
         with self.assertRaises((cmd.GPGError, FileNotFoundError)):
             with cmd.cmd_pipe(("python3", "--non-existing-option")) as proc:
-                proc.stdout.read()
+                cmd.assert_io(proc.stdout).read()
