@@ -1,5 +1,5 @@
 import unittest
-import typing
+from typing import Tuple, Dict, Optional, cast
 import sys
 from dataclasses import dataclass
 
@@ -10,15 +10,15 @@ deserialize = sys.modules["gpg_lite.deserialize"]
 
 
 class TestDeserialize(unittest.TestCase):
-    def test_dict_type(self):
-        self.assertEqual(deserialize.dict_type(typing.Dict[int, str]), (int, str))
+    def test_dict_type(self) -> None:
+        self.assertEqual(deserialize.dict_type(Dict[int, str]), (int, str))
         self.assertEqual(deserialize.dict_type(int), None)
 
-    def test_deserialize(self):
+    def test_deserialize(self) -> None:
         @dataclass
         class Y:
             x: int
-            y: typing.Tuple[bool]
+            y: Tuple[bool, ...]
 
         @dataclass
         class X:
@@ -34,13 +34,14 @@ class TestDeserialize(unittest.TestCase):
         @dataclass
         class Z:
             a: int
-            dct: typing.Dict[int, str]
+            dct: Dict[int, str]
 
-        data = {"a": 1, "dct": {1: "x", 2: "y"}}
+        inner = {1: "x", 2: "y"}
+        data = {"a": 1, "dct": inner}
         z = deserialize.deserialize(Z)(data)
 
-        self.assertEqual(z, Z(a=1, dct=data["dct"]))
+        self.assertEqual(z, Z(a=1, dct=inner))
 
         # typing.Optional
-        self.assertEqual(deserialize.deserialize(typing.Optional[int])(None), None)
-        self.assertEqual(deserialize.deserialize(typing.Optional[int])(1), 1)
+        self.assertEqual(deserialize.deserialize(cast(type, Optional[int]))(None), None)
+        self.assertEqual(deserialize.deserialize(cast(type, Optional[int]))(1), 1)
