@@ -7,7 +7,7 @@ from test.test_gpg import ENV_WITH_GPG
 @unittest.skipUnless(ENV_WITH_GPG, "Integration tests")
 class TestDocs(unittest.TestCase):
     @staticmethod
-    def test_gen_key():
+    def test_gen_key() -> None:
         """Read the python code snippets from README and exec"""
         read = False
         code = ""
