@@ -3,6 +3,7 @@ import os
 import tempfile
 import io
 from functools import partial
+from typing import Tuple, Any, cast, IO, Callable
 
 import gpg_lite as gpg
 from gpg_lite import keyserver, store
@@ -11,7 +12,6 @@ from gpg_lite.model import SignatureValidity
 
 ENV_WITH_GPG = os.environ.get("WITH_GPG", None) == "true"
 
-
 CN_UID = gpg.Uid(full_name="Chuck Norris", email="chuck.norris@roundhouse.gov")
 CN_FINGERPRINT = "CB5A3A5C2C4419BB09EFF879D6D229C51AB5D107"
 CN_SIGNATURE = gpg.Signature(
@@ -69,7 +69,6 @@ SGT_HARTMANN_SIGNATURE_REVOKED = gpg.RevocationSignature(
     comment="revsig\\nInvalid email",
 )
 
-
 UMLAUTS_UID = gpg.Uid(full_name="With Umläuts", email="with.umlauts@umlauts.info")
 UMLAUTS_FINGERPRINT = "F119E26211119BB09EFF879D6D229C51AB5D107C"
 MISSING_PUB_KEY_SIGNATURE = gpg.Signature(
@@ -112,12 +111,15 @@ CN_KEY = gpg.Key(
         AUTHORITY_SIGNATURE_REVOKED_FAKE,
     ),
     key_capabilities={
-        gpg.KeyCapability.Sign(True),
-        gpg.KeyCapability.Sign(False),
-        gpg.KeyCapability.Certify(True),
-        gpg.KeyCapability.Certify(False),
-        gpg.KeyCapability.Encrypt(True),
+        gpg.KeyCapability.sign,
+        gpg.KeyCapability.certify,
     },
+    combined_key_capabilities={
+        gpg.KeyCapability.sign,
+        gpg.KeyCapability.certify,
+        gpg.KeyCapability.encrypt,
+    },
+    disabled=False,
     key_type=gpg.KeyType.public,
     uids=(CN_UID,),
     owner_trust="u",
@@ -131,7 +133,7 @@ CN_KEY = gpg.Key(
             pub_key_algorithm=1,
             creation_date="1550241679",
             signatures=(CN_SIGNATURE,),
-            key_capabilities={gpg.KeyCapability.Encrypt(False)},
+            key_capabilities={gpg.KeyCapability.encrypt},
         ),
     ),
     origin=KEY_SERVER,
@@ -154,12 +156,15 @@ AUTHORITY_KEY = gpg.Key(
     creation_date="1565257866",
     signatures=(AUTHORITY_SIGNATURE,),
     key_capabilities={
-        gpg.KeyCapability.Sign(True),
-        gpg.KeyCapability.Sign(False),
-        gpg.KeyCapability.Certify(True),
-        gpg.KeyCapability.Certify(False),
-        gpg.KeyCapability.Encrypt(True),
+        gpg.KeyCapability.sign,
+        gpg.KeyCapability.certify,
     },
+    combined_key_capabilities={
+        gpg.KeyCapability.sign,
+        gpg.KeyCapability.certify,
+        gpg.KeyCapability.encrypt,
+    },
+    disabled=False,
     key_type=gpg.KeyType.public,
     uids=(AUTHORITY_UID,),
     owner_trust="-",
@@ -173,7 +178,7 @@ AUTHORITY_KEY = gpg.Key(
             pub_key_algorithm=1,
             creation_date="1565257866",
             signatures=(AUTHORITY_SIGNATURE,),
-            key_capabilities={gpg.KeyCapability.Encrypt(False)},
+            key_capabilities={gpg.KeyCapability.encrypt},
         ),
     ),
     origin=None,
@@ -181,7 +186,7 @@ AUTHORITY_KEY = gpg.Key(
 
 
 class TestUtils(unittest.TestCase):
-    def test_split_url(self):
+    def test_split_url(self) -> None:
         self.assertEqual(
             keyserver.split_host("http://hagrid.hogwarts.org:11371"),
             ("http://", "hagrid.hogwarts.org", ":11371"),
@@ -205,11 +210,11 @@ class TestUtils(unittest.TestCase):
         with self.assertRaises(ValueError):
             keyserver.split_host("hagrid.hogwar:ts.org")
 
-    def test_uid_missging_both_fields(self):
+    def test_uid_missging_both_fields(self) -> None:
         with self.assertRaises(ValueError):
             gpg.Uid(full_name=None, email=None)
 
-    def test_uid_from_str(self):
+    def test_uid_from_str(self) -> None:
         self.assertEqual(gpg.Uid.from_str("[User ID not found]"), None)
         self.assertEqual(gpg.Uid.from_str("[User-ID nicht gefunden]"), None)
         self.assertEqual(gpg.Uid.from_str("[Identité introuvable]"), None)
@@ -219,7 +224,7 @@ class TestUtils(unittest.TestCase):
         )
         self.assertEqual(
             gpg.Uid.from_str(
-                "Chuck Norris (Chucks's test PGP key) " "<chucknorris@roundhouse.gov>"
+                "Chuck Norris (Chucks's test PGP key) <chucknorris@roundhouse.gov>"
             ),
             gpg.Uid(
                 full_name="Chuck Norris (Chucks's test PGP key)",
@@ -235,7 +240,7 @@ class TestUtils(unittest.TestCase):
             gpg.Uid(full_name=None, email="chucknorris@roundhouse.gov"),
         )
 
-    def test_uid_repr(self):
+    def test_uid_repr(self) -> None:
         cases = (
             (
                 "Chuck Norris (Chucks's test PGP key)",
@@ -255,17 +260,7 @@ class TestUtils(unittest.TestCase):
 
 
 class TestGPG(unittest.TestCase):
-    def test_factory(self):
-        class TestFactory(store.Factory):
-            type_ = object
-
-        fct = TestFactory()
-        fct.append("keys", "A")
-        self.assertEqual(fct.kwargs, {"keys": ("A",)})
-        fct.append("keys", "B")
-        self.assertEqual(fct.kwargs, {"keys": ("A", "B")})
-
-    def test_parse_keys(self):
+    def test_parse_keys(self) -> None:
         # Verify parsing of GnuPG output for single and multiple key(s).
         for keys, payload in (
             ([CN_KEY], CN_KEY_PAYLOAD),
@@ -277,7 +272,7 @@ class TestGPG(unittest.TestCase):
             )
             self.assertEqual(parsed_keys, keys)
 
-    def test_parse_keys_revocation_sig(self):
+    def test_parse_keys_revocation_sig(self) -> None:
         (parsed_key,) = store.parse_keys(
             io.BytesIO(b"tru::1:1571400791:0:3:1:5\n" + CN_KEY_PAYLOAD)
         )
@@ -296,7 +291,7 @@ class TestGPG(unittest.TestCase):
             parsed_key.valid_signatures, [CN_SIGNATURE, AUTHORITY_SIGNATURE]
         )
 
-    def test_extract_key_id(self):
+    def test_extract_key_id(self) -> None:
         partial_msg = io.BytesIO(b"\x84\x8c\x03\xf3\rL3@z?c\x01\x03\xff]\xdf")
         pos = partial_msg.tell()
         (key_id,) = gpg.extract_key_id(partial_msg)
@@ -322,7 +317,7 @@ class TestGPG(unittest.TestCase):
             key_ids, ["D892C41917B20115", "32F9E2F80D52215E", "CCFAA7F412B06D83"]
         )
 
-    def test_extract_key_id_ascii_armored(self):
+    def test_extract_key_id_ascii_armored(self) -> None:
         partial_msg = io.BytesIO(b"-----BEGIN PGP MESSAGE-----\r\n\r\n")
         with self.assertRaises(gpg.GPGError) as cm:
             list(gpg.extract_key_id(partial_msg))
@@ -330,7 +325,7 @@ class TestGPG(unittest.TestCase):
             format(cm.exception), "ASCII armored encrypted data is not supported"
         )
 
-    def test_extract_key_id_from_sig(self):
+    def test_extract_key_id_from_sig(self) -> None:
         sig = (
             b"\x89\x023\x04\x00\x01\x08\x00\x1d\x16!\x04\xa8\x05h*\x13\xa1\x08\xca"
             b"\xab\x07\xef\xd0k\xe5(\xb1&\xd3\xe13\x05\x02_\x05\xcfP\x00\n\t\x10k"
@@ -358,7 +353,7 @@ class TestGPG(unittest.TestCase):
             b"\xe9\xea\xac\x00\x17\xe3Nf\xd6\xd1\xa0\xb9\x99\xe3[\xa1O?a\xd7\x06\x9fc"
         )
         key_id = gpg.extract_key_id_from_sig(sig)
-        self.assertEqual(key_id, "6BE528B126D3E133")
+        self.assertEqual(key_id, "A805682A13A108CAAB07EFD06BE528B126D3E133")
         sig = b"""-----BEGIN PGP SIGNATURE-----
 
 iQIzBAABCAAdFiEEqAVoKhOhCMqrB+/Qa+UosSbT4TMFAl8YCiwACgkQa+UosSbT
@@ -377,11 +372,45 @@ xERXJ/PqFoAktjU/mhNeDHovXdB+7378YkY3NHNSlFytYG0D5jo=
 -----END PGP SIGNATURE-----
 """
         key_id = gpg.extract_key_id_from_sig(sig)
-        self.assertEqual(key_id, "6BE528B126D3E133")
+        self.assertEqual(key_id, "A805682A13A108CAAB07EFD06BE528B126D3E133")
         key_id = gpg.extract_key_id_from_sig(sig.replace(b"\n", b"\r\n"))
-        self.assertEqual(key_id, "6BE528B126D3E133")
+        self.assertEqual(key_id, "A805682A13A108CAAB07EFD06BE528B126D3E133")
 
-    def test_capture_fpr(self):
+        # Signature generated with gpg 2.0.22
+        sig = (
+            b"\x89\x02\x1c\x04\x00\x01\x02\x00\x06\x05\x02a`,\xa6\x00\n\t\x10\xab"
+            b"\xd0chZI\x86\x83\xa2\xeb\x10\x00\x8e\xe6\x15\x0e\x9cf\xd6t\x07"
+            b"?f\x81\x100\x88\xa7\x94\x18\xf1j\x9c\xcc]\xb1D\x13\xd3A?"
+            b"\xb5(c\x85\xa9\xd86\xdc\x1fm\x99\xbb:D\x8b\x08D/\x85\xea"
+            b"\xe5\x17\xc5'\xca_8\x94\x00\x07\xb3\x96\xe6dJ\x0e\xed\xb7F,"
+            b"/\xaa\xb5\xaeB\x99\x17\xeb\xc9\xf4\xf1\xa5\xb0\x03\xe2l\x08\xf7\x8d\xfe"
+            b" \xee\x04x\xe7h\xbdv<\x98\x8d=\x84\xa6\xc8q\xd0wVJ"
+            b'\x1c\x83\x85\xf4o\xd0$ \xa543\xcd\xe5\x05\xc9\x10\xac\x8f"F'
+            b"\x1b\xfd\xa6<dKT\xd2\x13\x9e\xe1\x9bd\xc6Q\xb0n\xd2`Z"
+            b"\xda\x11\xdd\xe2\x95\xb6uY\xff\x8bA\xb1\xdd.\x0f|\xb1\x9d<c"
+            b"\xd0\x1f\xf3\x9f\xcdI\xb4\xb7\x05\xba\x0f\xc5\x1d\x9e\x0b\xeaT\xc76\xaa"
+            b"\x05f\xe3\xbe\xf2\xd6U\xc9sE?\xf9\xbd\xb2p0n\x9a@\xec"
+            b".\x9c'\xdb\x86{\x96?\x13|W\xbew[G\x94<\xc0\x8d\""
+            b"N\xe1C2\xed=\r\x8b\xb9]b\xb7q\xc6\xba\x8ev/8\xa1"
+            b"\xf6\xef\xc5J\xdb\xd7\xd2v\xeb\xbf\xd7\x18\x89\x95\xe3=`\xbf\x0c\xa8"
+            b"\xb4\xab\x0c\xcb\x07\xe2\xcc\xfa\xed\xe4MI\xec\x08\x94\n\x92\xd7\xfc\xbb"
+            b'\xc4\x18Gf\x9c{v\xbc\xb2"\xaaa.\xab\x03\xbb\xf3@\xcbg'
+            b"\xb5m\xe1\r`\xa8/\x98\xa6\x843\xd1\x80\xc3\xcdj\x9b6\x8c4"
+            b"G\xfc\xfe\x807S*\xf9Ts\xd2^\xb9\xa1\xe5(]V\xbf$"
+            b"\xf0\x89\xb3\xbaX\xcdL-\xa0\xca\xdc\tZA\x8e\xad).Q\xf2"
+            b"*\xc2t\x93\x8f\xff\xd6\xce\xddq\x14\xe1+gJ(V\xdd7\xd3"
+            b"\xed\x01\xcd\x9d\xbdPq\x87\x88\xb3\xc1\x8e\xed\xdc\x9bl\x0e\xb5\xbfI"
+            b"\x8c\x0b\xc2\x9cF\xbeib\x9av\xee~\xe59OvJ\xdc+0"
+            b"N\xd4\xd6\x0f,V\xda\xb9\x0b\xc0J\xfd\xb7l\xd1<\xd2\x0c\xd5\xba"
+            b"|9\xe0\x1fj19\xbc3\x93\x92t%\x10/<\xfc'\xa7\xe7"
+            b"\xf0\x8d\xad\x0c\xab\x9f`.\xdf@K\xef\xf2A\xca]+'\x9f\xab"
+            b"\xc7\xcd\xd9\x0b\xb4\x87:<\xef\x9a\xa4\x97XEvDi\x1a\x1a!"
+            b"\xf1\xa0\xf2"
+        )
+        key_id = gpg.extract_key_id_from_sig(sig)
+        self.assertEqual(key_id, "ABD063685A498683")
+
+    def test_capture_fpr(self) -> None:
         b = b"""
 [GNUPG:] NEWSIG
 gpg: Signature made Tue 10 Mar 2020 11:29:27 CET
@@ -439,7 +468,7 @@ gpg:          There is no indication that the signature belongs to the owner.
             ],
         )
 
-    def test_assert_mdc(self):
+    def test_assert_mdc(self) -> None:
         b = b"""
 [GNUPG:] ENC_TO AAAAAAAAAAAAA 1 0
 [GNUPG:] KEY_CONSIDERED AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA 0
@@ -483,20 +512,21 @@ gpg: Good signature from "Chuck Norris <chucknorris@roundhouse.gov>" [ultimate]
 
 
 class IntegrationTest(unittest.TestCase):
-    def _setup_key(self):
+    def _setup_key(self) -> gpg.Key:
         self.s.import_file(CN_SEC_KEY, passphrase="test")
         self.s.import_file(CN_PUB_KEY)
         (key,) = self.s.list_sec_keys()
         return key
 
-    def setUp(self):
+    def setUp(self) -> None:
         # pylint: disable=consider-using-with
         gpg_homedir = tempfile.TemporaryDirectory()
         self.gpg_homedir = gpg_homedir
+        # pylint: disable=unnecessary-dunder-call
         d = gpg_homedir.__enter__()
         self.s = gpg.GPGStore(d)
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         try:
             self.gpg_homedir.__exit__(None, None, None)
         except FileNotFoundError:
@@ -505,7 +535,7 @@ class IntegrationTest(unittest.TestCase):
 
 @unittest.skipUnless(ENV_WITH_GPG, "Integration tests")
 class TestGPGIntegration(IntegrationTest):
-    def test_revoke_key(self):
+    def test_revoke_key(self) -> None:
         if self.s._version < (2, 2, 0):  # pylint: disable=protected-access
             self.skipTest("Key revocation not supported for gnupg < 2.2.0")
         key = self._setup_key()
@@ -514,11 +544,28 @@ class TestGPGIntegration(IntegrationTest):
         (key,) = self.s.list_sec_keys()
         self.assertEqual(key.validity, gpg.Validity.revoked)
 
-    def test_list_keys(self):
+    def test_export_secret(self) -> None:
+        key = self._setup_key()
+        # no matching key for the given fingerprint
+        with self.assertRaises(GPGError) as cm:
+            self.s.export_secret("A" * 40, passphrase="test")
+        self.assertIn("not found", str(cm.exception))
+        # check if an ascii-armored key is returned
+        exported = self.s.export_secret(key.fingerprint, passphrase="test", armor=True)
+        self.assertTrue(exported.startswith(b"-----BEGIN PGP PRIVATE KEY BLOCK-----"))
+        self.assertTrue(exported.endswith(b"-----END PGP PRIVATE KEY BLOCK-----\n"))
+        # load the exported key and compare it with the original one
+        with tempfile.TemporaryDirectory() as tmp:
+            s = gpg.GPGStore(tmp)
+            s.import_file(exported, passphrase="test")
+            (imported_key,) = s.list_sec_keys()
+            self.assertEqual(imported_key, key)
+
+    def test_list_keys(self) -> None:
         self._setup_key()
         self.s.import_file(UMLAUTS_PUB_KEY)
 
-        def extract_keys(**kwargs):
+        def extract_keys(**kwargs: Any) -> Tuple[gpg.Key, gpg.Key]:
             keys = self.s.list_pub_keys(**kwargs)
             if keys[0].uids == (CN_UID,):
                 key_1, key_2 = keys
@@ -553,10 +600,10 @@ class TestGPGIntegration(IntegrationTest):
         keys = self.s.list_pub_keys(search_terms=(), sigs=True)
         self.assertEqual(keys, [])
 
-    def test_gen_key(self):
+    def test_gen_key(self) -> None:
         self.s.gen_key(
-            full_name=CN_UID.full_name,
-            email=CN_UID.email,
+            full_name=cast(str, CN_UID.full_name),
+            email=cast(str, CN_UID.email),
             passphrase="test",
             key_length=1024,
         )
@@ -569,12 +616,33 @@ class TestGPGIntegration(IntegrationTest):
         self.assertEqual(sec_key.key_length, 1024)
         self.assertEqual(sec_key.key_type, gpg.KeyType.secret)
 
-    def test_decrypt(self):
+    def test_gen_key_unprotected_ecc(self) -> None:
+        if self.s._version < (2, 2, 0):  # pylint: disable=protected-access
+            self.skipTest("ECC keys not supported for gnupg < 2.2.0")
+        self.s.gen_key(
+            full_name=cast(str, CN_UID.full_name),
+            email=cast(str, CN_UID.email),
+            passphrase=None,
+            key_length=1024,
+            key_type="eddsa",
+            subkey_type="ecdh",
+            key_curve="Ed25519",
+            subkey_curve="Curve25519",
+        )
+        # key length does not seem to be respected by ECC keys
+        (pub_key,) = self.s.list_pub_keys()
+        self.assertEqual(pub_key.uids, (CN_UID,))
+        self.assertEqual(pub_key.key_type, gpg.KeyType.public)
+        (sec_key,) = self.s.list_sec_keys()
+        self.assertEqual(sec_key.uids, (CN_UID,))
+        self.assertEqual(sec_key.key_type, gpg.KeyType.secret)
+
+    def test_decrypt(self) -> None:
         message = b"a"
         self.s.import_file(CN_SEC_KEY, passphrase="test")
         decrypted = io.BytesIO()
 
-        def capture_output(stream):
+        def capture_output(stream: IO[bytes]) -> None:
             decrypted.write(stream.read())
             decrypted.seek(0)
 
@@ -616,16 +684,19 @@ class TestGPGIntegration(IntegrationTest):
                 str(cm.exception), "Message integrity could not be verified."
             )
 
-    def test_encrypt_decrypt(self):
+    def test_encrypt_decrypt(self) -> None:
         key = self._setup_key()
         message = b"a"
         decrypted = io.BytesIO()
 
-        def capture_output(stream):
+        def capture_output(stream: IO[bytes]) -> None:
             decrypted.write(stream.read())
             decrypted.seek(0)
 
-        decrypt = partial(self.s.decrypt, output=capture_output, passphrase="test")
+        decrypt = cast(
+            Callable[[IO[bytes]], None],
+            partial(self.s.decrypt, output=capture_output, passphrase="test"),
+        )
         self.s.encrypt(
             source=message,
             output=decrypt,
@@ -634,13 +705,13 @@ class TestGPGIntegration(IntegrationTest):
         )
         self.assertEqual(decrypted.read(), message)
 
-    def test_encrypt_decrypt_compression(self):
+    def test_encrypt_decrypt_compression(self) -> None:
         key = self._setup_key()
         message = b"a" * 50
         compressed = io.BytesIO()
         not_compressed = io.BytesIO()
 
-        def capture_output(stream, buffer):
+        def capture_output(stream: IO[bytes], buffer: IO[bytes]) -> None:
             buffer.write(stream.read())
             buffer.seek(0)
 
@@ -670,7 +741,7 @@ class TestGPGIntegration(IntegrationTest):
                     recipients=(key.fingerprint,),
                     trust_model=gpg.TrustModel.always,
                     compress_algo=gpg.CompressAlgo.ZLIB,
-                    compress_level=compress_level,
+                    compress_level=cast(int, compress_level),
                 )
 
         # Check missing passphrase raises an error when signing is requested.
@@ -683,19 +754,19 @@ class TestGPGIntegration(IntegrationTest):
                 passphrase=None,
                 trust_model=gpg.TrustModel.always,
                 compress_algo=gpg.CompressAlgo.ZLIB,
-                compress_level=compress_level,
+                compress_level=cast(int, compress_level),
             )
 
         self.assertLess(len(compressed.read()), len(not_compressed.read()))
 
-    def test_delete_keys(self):
+    def test_delete_keys(self) -> None:
         self.s.import_file(SGT_HARTMANN_PUB_KEY)
         (key,) = self.s.list_pub_keys()
         self.assertEqual(key.fingerprint, SGT_HARTMANN_FINGERPRINT)
         self.s.delete_pub_keys(key.fingerprint)
         self.assertFalse(self.s.list_pub_keys())
 
-    def test_detach_sig(self):
+    def test_detach_sig(self) -> None:
         self.s.import_file(CN_SEC_KEY, passphrase="test")
         (cn_key,) = self.s.list_pub_keys()
         (cn_key,) = self.s.list_sec_keys()
@@ -707,6 +778,37 @@ class TestGPGIntegration(IntegrationTest):
         with self.assertRaises(gpg.GPGError):
             self.s.verify_detached_sig(sig=sig, src=b"wrong message")
 
+    def test_gnupg_home_dir_expansion(self) -> None:
+        input_paths = (
+            "/some/path",
+            "/some/path/",
+            "~/some/path/",
+            "/home/user/some/~/path",
+            "/home/user/some/../path",
+            "~/../some/path",
+            "",
+            None,
+        )
+        home_dir = os.path.expanduser("~")
+        default_gnupg_home_dir = store.get_default_gnupg_home_dir()
+        expected_values = (
+            "/some/path",
+            "/some/path",
+            os.path.join(home_dir, "some/path"),
+            "/home/user/some/~/path",
+            "/home/user/path",
+            os.path.join(os.path.dirname(home_dir), "some/path"),
+            default_gnupg_home_dir,
+            default_gnupg_home_dir,
+        )
+
+        # Note: the reason for using os.path.abspath() on the expected_value
+        # in the comparison below is to ensure the path uses the correct,
+        # plateform specific, path separator.
+        for input_path, expected_value in zip(input_paths, expected_values):
+            g = store.GPGStore(gnupg_home_dir=input_path)
+            self.assertEqual(g.gnupg_home_dir, os.path.abspath(expected_value))
+
 
 CN_PUB_KEY = b"""-----BEGIN PGP PUBLIC KEY BLOCK-----
 
@@ -774,7 +876,6 @@ T03X+D5LrOJJ857/vwL1Roi5UgjRjX1jMFV/tg ==
 -----END PGP PRIVATE KEY BLOCK-----
 """
 
-
 UMLAUTS_PUB_KEY = b"""-----BEGIN PGP PUBLIC KEY BLOCK-----
 
 mI0EXcQPZwEEAMFLJh5nhUL3e2cIIEHw2lUwleWu/FMgy5GVVGho1pzSjqxc2cML
